import random
import re


def generateWord(meaning, form, settings):
    '''Takes an English string, desired form, generation
    categories, settings, and optional form-specific rules.
    Returns a generated word.
    '''
    word = ""
    minS = settings["minS"]
    maxS = settings["maxS"]
    defaultrule = settings["rule"]
    formrules = settings["formrules"]
    categories = settings["categories"]

    if formrules is not None:
        if form in formrules:
            rule = formrules[form]
        else:
            rule = defaultrule
    else:
        rule = defaultrule

    for syllable in range(random.randint(minS, maxS)):
        word += generateSyllable(categories, rule)

    phonotactics = settings["phonotactics"]
    word = applyPhonotactics(word, phonotactics)

    if form in settings["form_phono"]:
        word = applyPhonotactics(word, settings["form_phono"][form])

    return {'english': meaning, 'word': word, 'form': form}


def isInt(s):
    try:
        int(s)
        return True
    except:
        return False


def generateSyllable(categories, rule):
    '''Takes a category dictionary and a rule. Returns a
    generated syllable.
    '''
    syllable = ""
    for place in rule:
        # Choose whether to include optional category
        if place[0] == "[":
            # If the optional category begins with a number (1-9), treat that
            # as the probability of the category being included. Otherwise,
            # default to 50%.

            if isInt(place[1]):
                probability = int(place[1])
                place = place[1:]
            else:
                probability = 2

            if random.randint(1, probability) == 1:
                category = chooseRandomCategory(place)
                if category in categories:
                    syllable += random.choice(categories[category])
                else:
                    syllable += category

        # Include mandatory list
        elif place[0] == "{":
            category = chooseRandomCategory(place)
            if category in categories:
                syllable += random.choice(categories[category])
            else:
                syllable += category

        # Include single category
        else:
            if place in categories:
                syllable += random.choice(categories[place])
            else:
                syllable += place

    return syllable


def chooseRandomCategory(rule):
    rule = rule[1:-1]
    categories = rule.split("/")
    return random.choice(categories)


def applyPhonotactics(word, phonotactics):
    '''Takes list of phonotactics rules and a word. Returns
    word with all rules applied.'''
    for rule in phonotactics:
        r = rule.split("->")
        word = re.sub(r[0], r[1], word)
    return word



import lexeme.Library as Library
import csv
import lexeme.IOHelper as IOHelper
import sys
import os
from tabulate import tabulate

wordgensettings = {}
formrules = {}
phonotactics = {}
formatString = ""


def add():
    '''Interface for addWord().'''
    word = {}
    word['english'] = input("Enter meaning in English: ")
    word['word'] = input("Enter word in conlang: ")

    forms = Library.getFieldOptions("form")
    forms.append("other")

    form = IOHelper.chooseOption("Enter word form",
                                forms)
    if form == "other":
        form = input("Enter new word form: ")

    word['form'] = form

    word = addCustomFields(word)
    Library.addWord(word)
    print("Word saved in database!")


def modify():
    '''Modify an existing word.'''
    try:
        conword = input("Enter word in conlang: ")
        if Library.wordExists(conlang=conword):
            word = Library.findConWord(conword, pop=False)
            outputWord(word)
        else:
            print("Word does not exist")
            return

        keys = list(word.keys())
        keys.remove("id")
        keys.append("NEW")
        keys.append("DELETE")

        another = True

        while another:
            key = IOHelper.chooseOption("Enter field to modify", keys)

            if key == "NEW":
                word = addCustomFields(word, prompt=False)
            elif key == "DELETE":
                keys.remove("NEW")
                keys.remove("DELETE")
                keys.remove("english")
                keys.remove("word")
                key = IOHelper.chooseOption("Enter field to delete", keys)
                keys.remove(key)
                del word[key]

                keys.insert(0, "english")
                keys.insert(0, "word")
                keys.append("NEW")
                keys.append("DELETE")
            else:
                if key in ["word", "english"]:
                    word[key] = input("Enter new value: ")
                else:
                    values = Library.getFieldOptions(key)
                    values.append("other")

                    v = IOHelper.chooseOption("Enter word value",
                                              values)

                    if v == "other":
                        v = input("Enter new value: ")

                    word[key] = v

            another = not IOHelper.yesNo("Finished modifying")

        # Delete word if finished modifying and add new word
        Library.findConWord(conword, pop=True)
        Library.addWord(word)

    except KeyboardInterrupt:
        pass


def listwords():
    '''Interface for listWords().'''
    t = IOHelper.chooseOption("Enter list type", ["all", "field"])

    if t == "field":
        fields = Library.getFields()

        f = IOHelper.chooseOption("Enter desired field", fields)

        options = Library.getFieldOptions(f)

        o = IOHelper.chooseOption("Enter option to list", options)

        l = Library.listWords(t, f, o)
    else:
        l = Library.listWords(t)

    if len(l) > 0:
        outputWordList(l)
    else:
        print("No words to display")


def quit():
    sys.exit(0)


def decline():
    ''' Allows user to select word to decline and declension, then outputs the
    declined word.
    '''
    word = input("Enter word (in conlang) to decline: ")

    try:
        result = Library.findConWord(word)
    except LookupError:
        print("Word not found in database")
        return 1

    prompt = "Select declension"
    dec = IOHelper.createMenu(prompt, Library.getAvailableDeclensions())

    output = Library.declineWord(result, dec)

    outputWord(output, "conlang")


def outputWordList(wordList):
    '''Take a list of words. Output list of words in table.'''
    table = []
    headers = ["English", "Conlang"]

    for word in wordList:
        row = []
        row.append(word["english"])
        row.append(word["word"])
        for item in word:
            if item not in ["english", "word", "id"]:
                row.append(word[item])
        table.append(row)

    for item in wordList[0]:
        if item not in ["english", "word", "id"]:
            headers.append(item.capitalize())

    print("")
    print(tabulate(table, headers=headers))
    print("")


def clearScreen():
    os.system("cls" if os.name == "nt" else "clear")
    lexeme = """
 _
| |    _____  _____ _ __ ___   ___
| |   / _ \ \/ / _ | '_ ` _ \ / _ |
| |__|  __/>  |  __| | | | | |  __/
|_____\___/_/\_\___|_| |_| |_|\___|
                                                """
    print(lexeme)


def outputWord(word, first="english"):
    '''Take word dictionary and optional first column. Output
    word in a table.
    '''
    table = [[], [], []]
    headers = []

    phonetic = Library.transcribePhonemes(word["word"])
    allophonetic = Library.transcribeAllophones(phonetic)

    if first == "english":
        table[0].append(word["english"])
        table[1].append("")
        table[2].append("")
        headers.append("English")

        table[0].append(word["word"])
        table[1].append(phonetic)
        table[2].append(allophonetic)
        headers.append("Conlang")
    elif first == "conlang":
        table[0].append(word["word"])
        table[1].append(phonetic)
        table[2].append(allophonetic)
        headers.append("Conlang")

        table[0].append(word["english"])
        table[1].append("")
        table[2].append("")
        headers.append("English")

    for item in word:
        if item not in ["word", "english", "id"]:
            table[0].append(word[item])
            table[1].append("")
            table[2].append("")
            headers.append(item.capitalize())

    print("")
    print(tabulate(table, headers=headers))
    print("")


def statistics():
    '''Interface for getStatistics().'''
    print("Words: " + str(Library.getStatistics()))


def search():
    '''Interface for searchWords().'''
    term = input("Enter search term: ")

    results = Library.searchWords(term)

    if len(results[0]) == 0 and len(results[1]) == 0:
        print("Word not found")
    else:
        for word in results[0]:
            outputWord(word, "english")
            print("")

        for word in results[1]:
            outputWord(word, "conlang")
            print("")


def batchgenerate():
    '''Run each word in file through generate.'''
    filename = input("Enter location of words file: ")
    try:
        with open(filename, "r") as f:
            for word in f:
                clearScreen()
                print("Generating word " + word.strip() + "...")
                generate(word.strip())
                input("Press enter to continue...")
    except FileNotFoundError:
        print("File not found! Double-check the path you are using.")
        return 1

    print("Finished batch generation!")


def importWords():
    '''Add words from csv file to database.'''
    filename = input("Enter location of word csv file: ")
    try:
        with open(filename, "r") as f:
            reader = csv.DictReader(f)
            for word in reader:
                Library.addWord(word)
            print("Words successfully imported!")

    except FileNotFoundError:
        print("File not found! Double-check the path you are using.")


def generate(english=None):
    '''Interface to generateWord().'''
    if english is None:
        english = input("Enter word in English: ")

    if Library.wordExists(english=english):
        print("Word already exists!")
        w = Library.findEnglishWord(english)
        outputWord(w)
        return 1

    forms = Library.getFieldOptions("form")
    forms.append("other")

    form = IOHelper.chooseOption("Enter word form",
                                 forms)

    if form == "other":
        form = input("Enter new word form: ")

    finalised = False
    while finalised is not True:
        word = Library.generateWord(english, form, wordgensettings)
        while Library.wordExists(conlang=word['word']):
            word = Library.generateWord(english, form, wordgensettings)
        #clearScreen()
        outputWord(word, "conlang")

        accepted = IOHelper.chooseOption("Accept word", ["y", "n", "e"])
        if accepted == "y":
            finalised = True
        elif accepted == "e":
            word['word'] = input("Enter modified word: ")
            finalised = True

    word = addCustomFields(word)

    Library.addWord(word)
    print("Word saved in database!")


def addCustomFields(word, prompt=True):
    '''Take word and allow user to set custom fields. Return
    completed word.
    '''

    if prompt:
        another = IOHelper.yesNo("Add custom field")
    else:
        another = True

    while another:
        options = Library.getFields()
        options.append("other")
        field = IOHelper.chooseOption("Enter desired field to add", options)
        if field == "other":
            new = input("Enter new field: ")
            value = input("Enter word value: ")
            word[new] = value
        else:
            values = Library.getFieldOptions(field)
            values.append("other")

            v = IOHelper.chooseOption("Enter word value",
                                      values)

            if v == "other":
                v = input("Enter new word value: ")

            word[field] = v

        another = IOHelper.yesNo("Add custom field")

    return word


def loadData(filename="default-config.txt"):
    '''Loads data from config file and passes it to Library.'''
    try:
        result = IOHelper.parseConfig(filename)
    except KeyError:
        print("Config file is malformed or does not exist")
        quit()

    phonemes = result[0]
    allophones = result[1]
    declensions = result[2]
    exportformat = result[4]

    global wordgensettings
    wordgensettings = result[3]

    global formatString
    formatString = exportformat

    Library.setPhonemes(phonemes)
    Library.setAllophones(allophones)
    Library.setDeclensions(declensions)

    return 0


def exportText():
    '''Interface for exportText().'''
    filename = input("Enter filename to export: ")
    Library.exportText(filename, formatString)


def export():
    '''Interface for exportWords().'''
    filename = input("Enter filename to export: ")
    Library.exportWords(filename)



import dataset
import re

from lexeme.Generator import generateWord

db = None
phonemes = {}
allophones = {}
declensions = {}


def transcribePhonemes(word):
    '''Transcribe from orthographic representation to phonetic
    representation.
    '''
    for current, new in phonemes.items():
        word = re.sub(current, new, word)

    word = "/" + word + "/"

    return word


def transcribeAllophones(word):
    '''Transcribe from phonetic representation to full IPA
    representation.
    '''
    word = word[1:-1]

    for current, new in allophones.items():
        word = re.sub(current, new, word)

    word = "[" + word + "]"
    return word


def getStatistics():
    '''Returns number of words in database.'''
    return len(db['words'])


def exportText(filename, formatString):
    '''Takes filename and outputs text file
    according to format specified in configuration
    file.'''
    fields = db['words'].columns
    fields.remove("id")

    strings = []

    for word in db['words'].all():
        s = formatString

        for field in fields:
            pattern = "{{" + field + "}}"

            if word[field] is not None:
                substitute = word[field]
            else:
                substitute = ""

            s = re.sub(pattern, substitute, s)

        strings.append(s + "\n")

    with open(filename, mode='w') as f:
        f.writelines(strings)


def exportWords(filename):
    '''Takes filename and outputs csv.'''
    allWords = db['words'].all()
    dataset.freeze(allWords, format='csv', filename=filename)
    print("Exported all words to " + filename)


def searchWords(term):
    '''Takes a search term. Returns tuple of two lists, the first
    populated with matching English words and the second with
    matching conlang words.
    '''
    englishresult = db['words'].find(english=term)

    conlangresult = db['words'].find(word=term)

    return (list(englishresult), list(conlangresult))


def getAvailableDeclensions():
    '''Returns declension list.'''
    return list(declensions)


def declineWord(word, d):
    '''Declines word with declension d. Returns declined word.'''
    dec = declensions[d].split("->")

    word['word'] = re.sub(dec[0], dec[1], word['word'])

    return word


def findConWord(term, pop=False):
    '''Finds the first occurrence of term in conlang column of database and
    returns as a word.
    '''
    word = db['words'].find_one(word=term)

    if word is None:
        raise LookupError

    if pop is True:
        db['words'].delete(word=term)

    return word


def findEnglishWord(term):
    '''Finds the first occurrence of term in English column of database
    and returns as a word.
    '''
    word = db['words'].find_one(english=term)

    if word is None:
        raise LookupError

    return word


def wordExists(english=None, conlang=None):
    '''Accepts string and searches for it in conlang words list and English words
    list. If word exists in database, returns True, otherwise returns False.
    '''
    if conlang is not None:
        try:
            findConWord(conlang)
            return True
        except LookupError:
            return False

    if english is not None:
        try:
            findEnglishWord(english)
            return True
        except LookupError:
            return False


def getFields():
    '''Returns list of fields, not including id, english, or word.'''
    fields = db['words'].columns
    fields.remove("english")
    fields.remove("word")
    fields.remove("id")

    return fields


def getFieldOptions(field):
    '''Takes a field. Returns all possible options for field that
    exist within database.
    '''
    l = list(db['words'][field])
    options = []

    for item in l:
        options.append(item[field])

    if None in options:
        options.remove(None)

    return options


def listWords(t, f=None, o=None):
    '''Takes type of list (full or specific form) and form. Returns list of
    matching words.
    '''
    outList = []

    if t == "all":
        for word in db['words']:
            outList.append(word)

    elif t == "field":
        q = 'SELECT * FROM words WHERE ' + f + ' LIKE "' + o + '"'
        for word in db.query(q):
            outList.append(word)

    return outList


def addWord(word):
    '''Takes word object and adds word to database.'''
    db['words'].insert(word)


def setPhonemes(l):
    global phonemes
    phonemes = l


def setAllophones(l):
    global allophones
    allophones = l


def setDeclensions(l):
    global declensions
    declensions = l


def loadDatabase(filename="words.db"):
    global db

    location = "sqlite:///" + filename
    db = dataset.connect(location)



import configparser


def parseConfig(filename):
    '''Takes path to config file.
    Returns phonemes, allophones, declension rules, dictionary
    of word generation categories, word generation settings,
    phonotactics rules, and string export settings.
    '''
    parser = configparser.ConfigParser()
    parser.read(filename)

    rawphonemes = convertList(parser["TRANSCRIPTION"]["Phonemes"])
    phonemes = convertListToDic(rawphonemes)
    rawallophones = convertList(parser["TRANSCRIPTION"]["Allophones"])
    allophones = convertListToDic(rawallophones)

    declensions = {}
    for rule in parser["DECLENSION"]:
        declensions[rule] = parser["DECLENSION"][rule]

    phonotactics = []
    for rule in parser["DEFAULT-PHONOTACTICS"]:
        phonotactics.append(parser["DEFAULT-PHONOTACTICS"][rule])

    wordgencategories = {}
    for cat in parser["WORDGEN-CATEGORIES"]:
        wordgencategories[cat] = convertList(parser["WORDGEN-CATEGORIES"][cat])

    wordsettings = {}
    wordsettings["categories"] = wordgencategories
    wordsettings["phonotactics"] = phonotactics
    wordsettings["maxS"] = int(parser["WORDGEN-SETTINGS"]["MaxSyllable"])
    wordsettings["minS"] = int(parser["WORDGEN-SETTINGS"]["MinSyllable"])
    wordsettings["rule"] = parseSylRule(parser["WORDGEN-SETTINGS"]["Rule"])

    # Parse form-specific rules
    if "FormRules" in parser["WORDGEN-SETTINGS"]:
        rawformrules = parser["WORDGEN-SETTINGS"]["FormRules"]
        formrules = convertListToDic(convertList(rawformrules))
        for rule in formrules:
            formrules[rule] = parseSylRule(formrules[rule])
    else:
        formrules = None

    wordsettings["formrules"] = formrules

    # Parse form-specific phonotactics
    form_phono = {}

    if "FORM-PHONOTACTICS" in parser:
        for form in parser["FORM-PHONOTACTICS"]:
            form_phono[form] = convertList(parser["FORM-PHONOTACTICS"][form])

    wordsettings["form_phono"] = form_phono

    formatString = parser["EXPORT"]["Format"]

    return (phonemes, allophones, declensions, wordsettings,
            formatString)


def parseSylRule(string):
    cats = string.split("|")
    rule = []
    for item in cats:
        if item[0] is not "(":
            rule.append(item.lower())
        else:
            i = item.split(")")
            templist = [i[0][1:].lower()]
            freq = int(i[1])
            for j in range(1, freq):
                templist.append(None)
            rule.append(templist)
    return rule


def convertList(string):
    '''Takes a list in string form such as "1, 2, 3" and returns
    a proper List.
    '''
    return string.replace(" ", "").split(",")


def convertListToDic(l):
    '''Takes a list of strings such as "'1:2', 'a:b'" and returns
    a dictionary.
    '''
    d = {}
    for item in l:
        isplit = item.split(":")
        d[isplit[0]] = isplit[1]
    return d


def parseDic(filename):
    '''Takes path to text file structured like so:
            example:8
            anotherexample:āçj
       And returns a dictionary with the first item of each line as the key and
       the second as the value.
    '''
    d = {}
    with open(filename, mode="r") as f:
        for line in f:
            l = line.strip()
            if l is not None:
                lsplit = l.split(":")
                d[lsplit[0]] = lsplit[1]
    return d


def parseList(filename):
    '''Takes path the text file structured like so:
            a
            b
            another value
       And returns a dictionary including each item.
    '''
    l = []
    with open(filename, mode="r") as f:
        for line in f:
            l.append(line.strip())
    return l


def createMenu(prompt, options):
    '''Takes a list of options and a prompt. Creates an input menu. Returns
    the item selected by the user.
    '''
    print(prompt + ":")
    for index, item in enumerate(options):
        bullet = "(" + str(index + 1) + ") "
        print(bullet + item)

    response = int(input("Enter selection: "))
    while response not in range(1, len(options) + 1):
        print("Selection not in range")
        response = int(input("Enter selection: "))

    return options[response - 1]


def chooseOption(prompt, options):
    olist = " ("
    for o in options[:-1]:
        olist = olist + o + "/"
    olist = olist + options[-1] + "): "

    response = input(prompt + olist)

    while response not in options:
        print("Response not in options")
        response = input(prompt + olist)

    return response


def yesNo(prompt):
    response = input(prompt + "? (y/n): ")
    while response not in ["y", "n"]:
        response = input(prompt + "? (y/n): ")

    if response == "y":
        return True
    else:
        return False



import cmd
import argparse
from lexeme.Interface import *


class Lexeme(cmd.Cmd):
    intro = "Welcome to Lexeme! Input '?' for help and commands."
    prompt = "\nEnter command: "

    def getHelp(self, command):
        with open("help.txt", "r") as f:
            line = f.readline().rstrip()
            while line != command:
                line = f.readline().rstrip()
            f.readline()
            while line != "====":
                print(line)
                line = f.readline().rstrip()

    def do_list(self, arg):
        clearScreen()
        try:
            listwords()
        except KeyboardInterrupt:
            pass

    def help_list(self):
        clearScreen()
        self.getHelp("LIST")

    def do_quit(self, arg):
        quit()

    def help_quit(self):
        clearScreen()
        self.getHelp("QUIT")

    def do_add(self, arg):
        clearScreen()
        try:
            add()
        except KeyboardInterrupt:
            pass

    def help_add(self):
        clearScreen()
        self.getHelp("ADD")

    def do_decline(self, arg):
        clearScreen()
        try:
            decline()
        except KeyboardInterrupt:
            pass

    def help_decline(self):
        clearScreen()
        self.getHelp("DECLINE")

    def do_modify(self, arg):
        clearScreen()
        modify()

    def help_modify(self):
        clearScreen()
        self.getHelp("MODIFY")

    def do_statistics(self, arg):
        clearScreen()
        statistics()

    def help_statistics(self):
        clearScreen()
        self.getHelp("STATISTICS")

    def do_search(self, arg):
        clearScreen()
        try:
            search()
        except KeyboardInterrupt:
            pass

    def help_search(self):
        clearScreen()
        self.getHelp("SEARCH")

    def do_generate(self, arg):
        clearScreen()
        try:
            generate()
        except KeyboardInterrupt:
            pass

    def help_generate(self, arg):
        clearScreen()
        self.getHelp("GENERATE")

    def do_export(self, arg):
        clearScreen()
        try:
            export()
        except KeyboardInterrupt:
            pass

    def help_export(self, arg):
        clearScreen()
        self.getHelp("EXPORT")

    def do_exportwords(self, arg):
        clearScreen()
        try:
            exportText()
        except KeyboardInterrupt:
            pass

    def help_exportwords(self):
        clearScreen()
        self.getHelp("EXPORTWORDS")

    def do_batch(self, arg):
        clearScreen()
        try:
            batchgenerate()
        except KeyboardInterrupt:
            pass

    def help_batch(self):
        clearScreen()
        self.getHelp("BATCH")

    def do_import(self, arg):
        clearScreen()
        try:
            importWords()
        except KeyboardInterrupt:
            pass

    def help_import(self):
        clearScreen()
        self.getHelp("IMPORT")

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--database", help="set database file")
    parser.add_argument("--config", help="set configuration file")
    args = parser.parse_args()

    if args.database is not None:
        Library.loadDatabase(args.database)

    else:
        Library.loadDatabase()

    if args.config is not None:
        loadData(args.config)
    else:
        loadData()

    clearScreen()

    Lexeme().cmdloop()

if __name__ == "__main__":
    main()



from distutils.core import setup
setup(
    name='lexeme',
    packages=['lexeme'],
    version='0.9.3',
    description='A conlang word database and generation program.',
    author='Cadel Watson',
    author_email='cadel@cadelwatson.com',
    url='https://github.com/kdelwat/Lexeme',
    # download_url='https://github.com/kdelwat/Lexeme/tarball/0.9.1',
    keywords=['testing'],
    classifiers=[],
    entry_points={
        'console_scripts': [
            'lexeme = lexeme.lexeme:main',
        ],
    },
    install_requires=[
        'dataset',
        'tabulate',
    ],
    package_data={'lexeme': ['default-config.txt']}
)
